xquery version "3.1";

module namespace kmgnav-utils="http://bdn-edition.de/ns/kmgnav-utils";
declare namespace bdn="http://www.bdn-edition.de/ns/1.0";
declare namespace tei="http://www.tei-c.org/ns/1.0";
(:declare namespace templates="http://exist-db.org/xquery/templates";:)
declare namespace tgmd="http://textgrid.info/namespaces/metadata/core/2010";
declare namespace html="http://www.w3.org/1999/xhtml";
declare namespace sade="https://mollenhauer-edition.de/ns/sade";

import module namespace templates="http://exist-db.org/xquery/templates";
import module namespace config="https://sade.textgrid.de/ns/config" at "../config.xqm";
import module namespace transformKMG="http://mollenhauer-edition.de/ns/transform" at "transform.xqm";

import module namespace console="http://exist-db.org/xquery/console";

(:
 Main template function for creating the second grey edition specific navigation bar
:)
declare function kmgnav-utils:nav-injection($node as node(), $model as map(*))
as element(div)
{
  <div id="edition-menu" class="edition-menu">
    <div class="center row edition-menu-content">
        <!-- current edition -->
        <div class="edition-menu-title column" aria-label="titledisplay">
          {templates:process(<span data-template="kmgnav-utils:get-title" />, $model)}
        </div>
        <ul class="edition-menu-controls row-s" role="list">
          <li class="row-s edition-menu-control">
            <button class="button-w-tooltip js-edition-menu__info" href="#" aria-labelledby="tooltip-info">
              <span class="icon-info" aria-hidden="true" />
            </button>
            <div role="tooltip" id="tooltip-info" class="bg-tertiary">Informationen und Metadaten anzeigen</div>
          </li>
          {kmgnav-utils:downloads()}
          <li class="row-s edition-menu-control">
            <nav class="nav-button">
              <button class="button-w-tooltip dropdown"  aria-haspopup="true" aria-expanded="false" aria-labelledby="tooltip-views">
                <span aria-hidden="true" class="icon-layers" />
              </button>
              <div role="tooltip" id="tooltip-views" class="bg-tertiary">Editionsansichten auswählen</div>
              {templates:process(<span data-template="kmgnav-utils:make-views-content" />, $model)}
            </nav>
          </li>
          {kmgnav-utils:work-overview()}
          {kmgnav-utils:pages-overview()}
          <li class="row-s edition-menu-control">
            <a id="fullscreen-toggle" class="button-w-tooltip" href="#" role="button" aria-labelledby="tooltip-fullscreen">
              <span class="icon-fullscreen" aria-hidden="true" />
            </a>
            <div role="tooltip" id="tooltip-fullscreen" class="bg-tertiary">
              Vollbildmodus
              <span class="fullscreen-open">öffnen</span>
              <span class="fullscreen-close hidden">schließen</span>
            </div>
          </li>
        </ul>

    </div>
  </div>
};

(:~
 : Creates the content for the views panel of the text
 : (e.g. diplomatic view, synoptic view)
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @author Johannes Biermann
 : @author Simon Sendler
 : @version 1.1
 : @param $model The model of the current page
 : @return element(span) Containing the body content
 :)
declare function kmgnav-utils:make-views-content($node as node()*, $model as map(*))
as element(ul)*
{
    let $filename := request:get-parameter("file","")
    let $file := doc($config:sade-data || $filename || ".xml")
    let $edts-tmp := string-join($file//tei:witness[(fn:exists(@xml:id) and not(@sade:singleview/string() = "false"))]/@xml:id, " ")
    let $editions := distinct-values(tokenize($edts-tmp, "[#\s]+"))
    (: makes iterating easier :)
    let $desc := string-join($file/tei:TEI/tei:teiHeader/tei:fileDesc/tei:sourceDesc/tei:listWit/tei:witness[(fn:exists(@xml:id) and not(@sade:singleview/string() = "false"))]/tei:desc, "#")
    let $baseText := $file//tei:witness[1]/@xml:id/string()

    return
      element ul
      {
        attribute class
        {
          "hidden bg-primary edition-menu-content"
        },
        attribute role { "menu" },
        if ( count($editions) > 1 ) then
          (element li
          {
            attribute class
            {
              if (kmgnav-utils:isViewActive("digital")) then "current-edition" else ""
            },
            element a
            {

              attribute href { kmgnav-utils:make-url($filename, "digital") },
              element span
              {
                "Textkritische interaktive Ansicht mit "|| $baseText ||" als Leittext"
              }
            }
          },
          element div
          {
            attribute class { "dropdown-divider" }
          })
        else
          (),
        for $edition at $countEdt in $editions
            for $date at $countDate in tokenize($desc, "#")
            where $edition != "" and $countEdt = $countDate
            return
              element li
              {
                attribute class
                {
                  if (kmgnav-utils:isViewActive($edition)) then "current-edition" else ""
                },
                element a
                {

                  attribute href {
                    kmgnav-utils:make-url($filename, $edition)
                  },
                  element span
                  {
                    string-join(("Textfassung", $edition, $date), " ")
                  }
                }
              },
          element div
          {
            attribute class { "dropdown-divider" }
          },
          element li
          {
            attribute class
            {
                if (kmgnav-utils:isViewActive("commentary")) then "current-edition" else ""
            },
            element a
              {
                  attribute href { kmgnav-utils:make-url($filename, "commentary") },
                  element span
                  {
                      "Werkkommentar"
                  }
              }
          }
      }
};


(:~
 : Creates links to kmg.html with all necessary information,
 : i.e. file name and edition view.
 :
 : @author Hannes Riebl
 : @author Michelle Weidling
 : @author Johannes Biermann
 : @version 1.0
 : @param $filename The current file
 : @param $edition The current edition view
 : @param $foreword true() if we want to set a link to the editorial foreword
 : @return The link as xs:string
 :)
declare function kmgnav-utils:make-url($filename as xs:string, $edition as xs:string) as xs:string*
{
        let $url :=
            $config:app-root || "/kmg.html?"
            => substring-after("sade")
        return
            "kmg.html?&#38;file=" || $filename || "&#38;edition=" || $edition
};

(:~
 : Checks if the current view (request parameter edition) matches a certain view
 : can be used to highlight the current view in the navigation as active
 :
 : @author Johannes Biermann
 : @version 1.0
 : @param edition The edition/view to display
 : @return true false depending on if the edition matches the current request paramater
 :)
declare function kmgnav-utils:isViewActive($edition as xs:string) as xs:boolean
{
    let $activeEdition := request:get-parameter("edition","a")

    let $isActive :=
         if (compare($activeEdition, $edition) = 0)
         then
            true()
         else
            false()

     return $isActive
};

(:~
 : Creates a span that displays the current author and his work's title. This
 : html:span is placed in the edition's navigation bar.
 :
 :)
declare function kmgnav-utils:get-title($node as node(), $model as map(*))
as element(span)+
{
    let $filename := request:get-parameter("file","")
    let $doc := doc($config:sade-data || $filename || ".xml")
    let $baseText := $doc//tei:witness[1]/@xml:id/string()

    let $activeEdition := request:get-parameter("edition","a")
    let $edition-display :=
        switch ( $activeEdition)
          case "digital" return
            " [Textkritische interaktive Ansicht mit " || $baseText || " als Leittext]"
          case "commentary" return
            " [Werkkommentar]"
          default return
            " [Textfassung " ||$activeEdition || "]"
    let $title := string-join(transformKMG:transform($doc//tei:title[@type="main"], $filename, "none", "", ""), " ")

    return
        (<span>{$title}</span>,
        <span>{$edition-display}</span>)
};

declare function kmgnav-utils:work-overview()
as element(li)?
{
  let $activeEdition := request:get-parameter("edition","a")
  let $filename := request:get-parameter("file", "")
  let $doc := doc($config:edition || $filename || "-" || $activeEdition || ".html")
  let $headers := $doc//html:span[@class='toc-anchor']
  return
    if ( $headers) then
    <li class="row-s edition-menu-control">
      <nav class="nav-button">
        <button class="button-w-tooltip dropdown"  aria-haspopup="true" aria-expanded="false" aria-labelledby="tooltip-internalnav">
          <span class="icon-bookmark" aria-hidden="true" />
        </button>
        <div role="tooltip" id="tooltip-internalnav" class="bg-tertiary">Zu Abschnitt springen</div>
        {kmgnav-utils:make-work-overview($headers)}
      </nav>
    </li>
  else
    ()
};

(:~~~~~~~~~~~~~~~~~~~~~~~~
   Creates Overview of chapters
 : @author Florian Barth
 : @author Markus Matoni
 : @author Simon Sendler
 :)

declare function kmgnav-utils:make-work-overview($headers as node()+)
as element(ul)
{
  element ul
  {
    attribute class { "bg-primary hidden edition-menu-content edition-toc" },
    attribute role { "menu" },
    for $header in $headers return
      element li
      {
        element a
          {
            attribute href {"#" || $header/@id/string()},
            element span {
                attribute class {"text--highlight"},
                normalize-space($header/@data-title/string())
            }
          }
      }
  }
};

declare function kmgnav-utils:pages-overview()
as element(li)?
{
  let $activeEdition := request:get-parameter("edition","a")
  let $filename := request:get-parameter("file", "")
  let $doc := doc($config:edition || $filename || "-" || $activeEdition || ".html")
  let $pages := $doc//html:span[@class='page-anchor']
  return
    if ( $pages) then
    <li class="row-s edition-menu-control">
      <nav class="nav-button">
        <button class="button-w-tooltip dropdown"  aria-haspopup="true" aria-expanded="false" aria-labelledby="tooltip-internalnav">
          <span class="fa fa-book-open" aria-hidden="true" />
        </button>
        <div role="tooltip" id="tooltip-internalnav" class="bg-tertiary">Zu Seite springen</div>
        {kmgnav-utils:make-pages-overview($pages, $activeEdition)}
      </nav>
    </li>
  else
    ()
};

(:~~~~~~~~~~~~~~~~~~~~~~~~
   Creates Overview of chapters
 : @author Florian Barth
 : @author Markus Matoni
 : @author Simon Sendler
 :)

declare function kmgnav-utils:make-pages-overview($pages as node()+, $edition as xs:string)
as element(ul)
{
  element ul
  {
    attribute class { "bg-primary hidden edition-menu-content edition-toc" },
    attribute role { "menu" },
    if ( $edition = "digital") then
      for $page in $pages
      where not($page/ancestor::html:span[@class=rdg-digital])
      return
        element li
        {
          element a
            {
              attribute href {"#" || $page/@id/string()},
              element span {
                  attribute class {"text--highlight"},
                  normalize-space($page/@data-wit/string() || $page/@data-title/string())
              }
            }
        }
    else
      for $page in $pages
      return
        element li
        {
          element a
            {
              attribute href {"#" || $page/@id/string()},
              element span {
                  attribute class {"text--highlight"},
                  normalize-space($page/@data-title/string())
              }
            }
        }
  }
};

declare function kmgnav-utils:downloads()
as element(li)
{
  let $doc := doc($config:sade-data || request:get-parameter("file", "") || ".xml")
  let $pdf := $doc//tei:teiHeader//tei:xenoData/tei:relatedResource[@type="pdf"]/@uri/string()
  return
  element li {
    attribute class {"row-s edition-menu-control"},
    element button {
      attribute class {"button-w-tooltip dropdown"},
      element span {
        attribute aria-hidden {"true"},
        attribute class {"icon-download"}
      }
    },
    element div {
      attribute role {"tooltip"},
      attribute id {"tooltip-downlaod"},
      attribute class {"bg-tertiary"},
      "Downloads"
    },
    element ul {
      attribute class { "bg-primary hidden edition-menu-content" },
      attribute role { "menu" },
      element li {
        element a {
          attribute href {"api/download/xml/" || request:get-parameter("file", "")},
          attribute target {"_blank"},
          "TEI-XML"
        }
      },
      element li {
        element a {
          attribute href {"api/download/plain/" || request:get-parameter("file", "") || "/" || request:get-parameter("edition", "a")},
          attribute target {"_blank"},
          "Plain Text"
        }
      },
      if ($pdf) then
        element li {
          element a {
            attribute href {$pdf},
            attribute target {"_blank"},
            "PDF"
          }
        }
      else
        ()
    }
  }
};
