xquery version "3.1";

module namespace search="http://mollenhauer-edition.de/ns/search";

declare namespace tei="http://www.tei-c.org/ns/1.0";
declare namespace html="http://www.w3.org/1999/xhtml";
declare namespace index="https://mollenhauer-edition.de/index";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";
import module namespace kwic="http://exist-db.org/xquery/kwic";

import module namespace facets="http://mollenhauer-edition.de/ns/facets" at "facets.xqm";
import module namespace options="http://mollenhauer-edition.de/ns/options" at "options.xqm";
import module namespace lang="https://sade.textgrid.de/ns/lang" at "../../lang.xqm";

declare function search:search-area($term as xs:string*) {
    let $search := search:search($term)
    return
        element div {
            attribute class {"search-area"},
            element div {
                attribute class {"search-params"},
                options:get-options($search),
                facets:get-facets($search)
            },
            element div {
                attribute class {"search-results"},
                search:pagination($search),
                search:results($search, $term)
            }
        }

};

declare function search:pagination($search)
{
    let $count := count(
        for $hits in $search return
        for $hit in (kwic:summarize($hits, <config width="1"/>)) return $hit
    )
    let $text-count := count($search)
    let $text-hits := lang:translate("hits") || " in"
    let $text-texts := if ($text-count eq 1) then lang:translate("textSg") else lang:translate("textPlDat")
    return
    element div {
        attribute class { "search-pagination" },
        element span {
            attribute class { "search-pagination-count" },
            $count, $text-hits, $text-count, $text-texts
        }
    }
};

declare function search:results($search, $term as xs:string)
{
    element ol {
        attribute class {"search-list search-results"},
        attribute role {"list"},
        for $hits in $search
        let $doc-name := util:document-name($hits)
        let $uri := $doc-name => substring-before("-")
        let $view := $doc-name => substring-before(".") => substring-after("-")
        let $doc := doc('/db/data/sade/textgrid/data/' || $uri || '.xml')
        let $title := $hits/@data-title/string()
        return
            element ol {
                attribute class {"search-list"},
                attribute role {"list"},
                element h2 {
                    attribute class {"search-result-title"},
                    $title || " (" || lang:translate($view) || ")"
                },
                for $hit in kwic:summarize($hits, <config width="40"/>)
                order by ft:score($hits) descending
                return
                    element li {
                        attribute class {"search-result-hit"},
                        element a {
                            attribute class {"search-result"},
                            attribute href {"kmg.html?file="
                                || $uri
                                || "&amp;edition="
                                || $view
                                || "&amp;search="
                                || $term
                                },
                            attribute target {"_blank"},
                            element span {
                                attribute class {"fa fa-arrow-right"}
                            },
                            (: hits are returned as html:p, but we only want the contained spans :)
                            $hit/*
                        }
                    }
            }
    }
};

declare function search:search($term as xs:string)
{
    let $facets := search:facet-map()
    let $options :=
        map {
            "facets": $facets,
            "default-operator": "and"
        }
    let $query := $term
        (: element query {
            element phrase { $term }
        } :)
    return
        collection('/db/data/sade/resources/edition')/html:article[ft:query(., $query, $options)]
};

(:~
 : This function maps the search parameters to the relevant facets.
 : This is mostly straightforward, but the "view" and "main-wit" parameters requires some explicit handling,
 : as it is presented not as a facet, but as a set of options in the front end.
 :)
declare function search:facet-map()
as map(*)
{
    let $genre := request:get-parameter("genre", "") => tokenize("\+")
    let $keyword := request:get-parameter("keyword", "") => tokenize("\+")
    let $wit := request:get-parameter("wit", "") => tokenize("\+")
    let $view := request:get-parameter("view", "") => tokenize("\+")
    let $digital :=
        if ($view  => filter(function($param as xs:string) {contains($param, "digital")}))
        then "digital"
        else ()
    let $commentary :=
        if ($view  => filter(function($param as xs:string) {contains($param, "commentary")}))
        then "commentary"
        else ()
    let $all :=
        if ($view  => filter(function($param as xs:string) {contains($param, "other")}))
        then
            "other"
        else ()
    let $view-facet := ("main", $digital, $commentary, $all)
    return
    map {
        "view": $view-facet,
        "genre": $genre,
        "keyword": $keyword,
        "wit": $wit
    }
};